/*****************************************************************************
 *
 * File: Cmn7_4.ddl
 * Version: 7.4.0
 *
 *****************************************************************************
 * Copyright (c) 2010, HART Communication Foundation, All Rights Reserved
 *****************************************************************************
 *
 * Description: This file contains the DDL based model of the Common Practice
 *      Command Soecification Rev.8.0.  There were significant changes in this 
 *      model reflecting the increasing focus on the device variable. This 
 *      resulted in a hierachal model that matches the one taught in the 
 *      Writing Device Description Workshop and the Command Summary 
 *      Specification.
 *
 * Source Specifications
 *      Command Practice Command Specification      Rev.7.1
 *      Command Summary Specification               Rev.7.1
 *      Universal Command Specification             Rev.5.2
 *      Common Tables Specification                 Rev.13.0
 *
 *****************************************************************************
 */

/*****************************************************************************
 * The manufacturer line
 *****************************************************************************
 */
MANUFACTURER  STANDARD, DEVICE_TYPE _COMMON_PRACTICE, DEVICE_REVISION 7, DD_REVISION 4

/*****************************************************************************
 * Include Files
 *****************************************************************************
 */
#include "macros.h"
#include "methods.h"

/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */
#define UPDATE_IN_PROGRESS      8
#define DEVICE_BUSY             32
#define MAXIMUM_RESET_RETRIES   10

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.1
 * Data Link Layer Commands.  The commands affect the operation of the Data
 * Layer or Enhances its capabilities
 *
 * Commands 59
 */

COMMAND write_number_of_response_preambles
{
    NUMBER 59;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            response_preambles
        }
        REPLY
        {
            response_code, device_status, response_preambles
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.2
 * PV Range Commands.  The commands affect the scaling of the digital value
 * to/from the loop current
 *
 * Commands 35, 36, 37
 */

/*
 *****************************************************************************
 * Write PV Range Values  changes the scaling relationship between the loop
 * current and the PV Digital Value
 *
 *NOTE: As per the Protocol Specification the Engineering Units Can be different
 *  from the PV Units.  The Units sent to the device must not affect the PV 
 *  Units Use Command 44 for this purpose
 */
COMMAND write_pv_range_values
{
    NUMBER 35;
    OPERATION WRITE;
    TRANSACTION 
    {
        REQUEST
        {
            PV.RANGING.RANGE_UNITS (INFO), PV.RANGING.UPPER_RANGE_VALUE, PV.RANGING.LOWER_RANGE_VALUE
        }
        REPLY
        {
            response_code, device_status,
            PV.RANGING.RANGE_UNITS (INFO), PV.RANGING.UPPER_RANGE_VALUE, PV.RANGING.LOWER_RANGE_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR,   [invalid_selection];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        9,  DATA_ENTRY_ERROR,   [lower_range_value_too_high];
        10, DATA_ENTRY_ERROR,   [lower_range_value_too_low];
        11, DATA_ENTRY_ERROR,   [upper_range_value_too_high];
        12, DATA_ENTRY_ERROR,   [upper_range_value_too_low];
        13, DATA_ENTRY_ERROR,   [URV_and_LRV_out_of_limits];
        14, DATA_ENTRY_WARNING, [span_too_small];
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy]; 
    }
}

/*
 *****************************************************************************
 * This Command sets the URV to the current conditions.  Sets the 100 Percent
 * Range operating point
 */
COMMAND set_pv_upper_range_value
{
    NUMBER 36;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        9,  PROCESS_ERROR,      [applied_process_too_high];
        10, PROCESS_ERROR,      [applied_process_too_low];
        14, DATA_ENTRY_WARNING, [span_too_small];
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
    }
}

/*
 *****************************************************************************
 * This Command sets the URV to the current conditions.  Sets the 0 Percent
 * Range operating point
 */
COMMAND set_pv_lower_range_value
{
    NUMBER 37;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        9,  PROCESS_ERROR,      [applied_process_too_high];
        10, PROCESS_ERROR,      [applied_process_too_low];
        14, DATA_ENTRY_WARNING, [new_LRV_pushed_URV_over_USL];
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
    }
}

METHOD applied_rerange
{
    LABEL [apply_values];
    HELP [applied_values_help];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long varid[3];
        int choice;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        varid[0] = VARID(PV.DEVICE_VARIABLE.DIGITAL_VALUE);
        varid[1] = VARID(PV.DEVICE_VARIABLE.DIGITAL_UNITS);

        while (1)
        {
            get_dictionary_string(set_the, disp_string1, slen1);
            get_dictionary_string(choice_4_20_exit, disp_string2, slen2);

            switch (SELECT_FROM_LIST(disp_string1, disp_string2))
            {
                case 0:
                    get_dictionary_string(apply_new_4ma_input, disp_string1,slen1);
                    ACKNOWLEDGE(disp_string1);

                    do 
                    {
                        send_command(1);

                        get_dictionary_string(current_applied_process_val, disp_string1, slen1);
                        get_dictionary_string(set_4ma_read_new_leave, disp_string2, slen2);

                        choice = select_from_list(disp_string1, varid, disp_string2);

                    } while (choice == 1);

                    if (choice == 0)
                    {
                        send(37, status);

                        if (status[STATUS_RESPONSE_CODE])
                        {
                            display_response_status(37, status[STATUS_RESPONSE_CODE]);
                        }

                        send_command(15);
                    }
                    break ;

                case 1:
                    get_dictionary_string(apply_new_20ma_input, disp_string1, slen1);
                    ACKNOWLEDGE(disp_string1);

                    do
                    {
                        send_command(1);

                        get_dictionary_string(current_applied_process_val, disp_string1, slen1);
                        get_dictionary_string(set_20ma_read_new_leave, disp_string2, slen2);

                        choice = select_from_list(disp_string1, varid,disp_string2);

                    } while (choice == 1);

                    if (choice == 0)
                    {
                        send(36,status);

                        if (status[STATUS_RESPONSE_CODE])
                        {
                            display_response_status(36, status[STATUS_RESPONSE_CODE]);
                        }

                        send_command(15);
                    }
                    break;

                case 2:
                    process_abort();

                default:
                    process_abort();
            }
        }
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.3
 * Loop Current Commands
 *
 * Commands 40, 45, 46
 */

COMMAND enter_exit_fixed_pv_current_mode
{
    NUMBER 40;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            PV.DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, PV.DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        11, MODE_ERROR,       [in_multidrop_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND trim_pv_current_dac_zero
{
    NUMBER 45;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            PV.DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, PV.DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        9,  MODE_ERROR,       [not_in_proper_current_mode];
        11, MODE_ERROR,       [in_multidrop_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND trim_pv_current_dac_gain
{
    NUMBER 46;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            PV.DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, PV.DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        9,  MODE_ERROR,       [not_in_proper_current_mode];
        11, MODE_ERROR,       [in_multidrop_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}
 
METHOD transmitter_loop_test
{
    LABEL [loop_test];
    HELP [loop_test_help];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[1];
        int use_abort_method;
        int choice;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        /* 
         * Here comes the standard part for analog outputs, e.g. transmitters 
         */
        use_abort_method = 0;

        do 
        {
            get_dictionary_string(choose_analog_output_level, disp_string1, slen1);
            get_dictionary_string(choices_4_20_other_end, disp_string2, slen2);

            choice = SELECT_FROM_LIST(disp_string1, disp_string2);

            switch (choice)
            {
            case 0:
                assign_float(PV.DAQ.ANALOG_VALUE,4.0);
                break;

            case 1:
                assign_float(PV.DAQ.ANALOG_VALUE,20.0);
                break;

            case 2:
                get_dictionary_string(loop_output, disp_string1, slen1);
                GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);
                break;

            case 3:
            default:
                process_abort();
                break;
            }

            send(40, status);

            if (!use_abort_method)
            {
                if ((status[STATUS_RESPONSE_CODE] != 0)
                        && ( status[STATUS_RESPONSE_CODE] != 3)
                        && ( status[STATUS_RESPONSE_CODE] != 4))
                {
                    display_response_status(40,status[STATUS_RESPONSE_CODE]);
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }

                if (status[STATUS_RESPONSE_CODE] == 0)
                {
                    remove_abort_method(return_to_normal);
                    add_abort_method(leave_fixed_current_mode);
                    use_abort_method =1;
                }
            }

            if (status[STATUS_RESPONSE_CODE])
            {
                /* 
                 * Display the resonse code from the error 
                 */
                display_response_status(40, status[STATUS_RESPONSE_CODE]);

                /* 
                 * If the response code is either 3 or 4, meaning the value
                 * was either too large or too small, do not abort method.
                 * Allow the user to try again. 
                 */ 
                if ((status[STATUS_RESPONSE_CODE] != 3)
                        && ( status[STATUS_RESPONSE_CODE] != 4))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }
            }
            else
            {
                var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);
                get_dictionary_string(fld_dev_otpt_fixed, disp_string1, slen1);
                acknowledge(disp_string1, var_id);
            }
        } while (choice < 3);

        process_abort();
    }
}

METHOD transmitter_dac_trim
{
    LABEL [DAC_trim];
    HELP [digital_to_analog_trim_help];
/*  APPHELP "This type of calibration is usually used to compensate for"
            " drifting or aging output electronics"; */
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[1];
        int retry;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(connect_reference_meter, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(setting_fld_dev_4_ma, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        assign_float(PV.DAQ.ANALOG_VALUE,4.0);
        send(40, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(40,status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        remove_abort_method(return_to_normal);
        add_abort_method(leave_fixed_current_mode);

        var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);

        do
        {
            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);

            send(45, status);

            assign_float(PV.DAQ.ANALOG_VALUE, 4.0);  /*Put the previous value back */

            if (( status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_otpt_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        get_dictionary_string(setting_fld_dev_20_ma, disp_string1,slen1);
        ACKNOWLEDGE(disp_string1);

        assign_float(PV.DAQ.ANALOG_VALUE, 20.0);

        send(40, status);

        if ( status[STATUS_RESPONSE_CODE])
        {
            display_response_status(40, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        do
        {
            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);

            send(46,status);

            assign_float(PV.DAQ.ANALOG_VALUE, 20.0);  /*Put the previous value back */

            if ((status[STATUS_RESPONSE_CODE] == 3 ) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if ( status[STATUS_RESPONSE_CODE])
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_otpt_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        process_abort() ;
    }
}

METHOD valve_loop_test
{
    LABEL [loop_test];
    HELP [valve_loop_test_help];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[77];
        char disp_string2[60];
        long var_id[1];
        int use_abort_method;
        int choice;
        int slen1;
        int slen2;
        float current;

        slen1 = 77;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);

        /* 
         * Here comes the standard part for analog inputs, e.g. valves 
         */
        do
        {
            get_dictionary_string(choose_analog_input_level, disp_string1, slen1);
            get_dictionary_string(choices_4_20_other_end, disp_string2, slen2);

            choice = SELECT_FROM_LIST(disp_string1, disp_string2);

            switch (choice)
            {
            case 0:
                current = 4.0;
                get_dictionary_string(apply_current_input_to_value, disp_string1, slen1);
                break;

            case 1:
                current = 20.0;
                get_dictionary_string(apply_current_input_to_value, disp_string1, slen1);
                break;

            case 2:
                get_dictionary_string(apply_current_input_to_desired_value, disp_string1, slen1);
                break;

            case 3:
            default:
                process_abort();
                break;
            }

            send(2, status);     /*Test the device to see if all is ok */

            if ((status[STATUS_RESPONSE_CODE] != 0) && (status[STATUS_RESPONSE_CODE] != 8))
            {
                /* 
                 * Only abort if the response code something value other
                 * than 0 (sucess) or 8 (warning)  
                 */
                display_response_status(2, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }

            acknowledge(disp_string1, var_id); /* disp_string1 is defined in switch/case above */

        } while (choice < 3);

        process_abort();
    }
}

METHOD valve_adc_trim
{
    LABEL [adc_trim];
    HELP [analog_to_digital_trim_help];
/*  APPHELP "This type of calibration is usually used to compensate for"
            " drifting or aging output electronics"; */
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[77];
        char disp_string2[60];
        long var_id[1];
        int retry;
        int slen1;
        int slen2;
        float current;

        slen1 = 77;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(connect_reference_meter, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);

        /*
         * 4 mA test
         */
        current = 4.0;

        send(2, status);     /*Test the device to see if all is ok */

        if ((status[STATUS_RESPONSE_CODE] != 0) && (status[STATUS_RESPONSE_CODE] != 8))
        {
            display_response_status(2, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        get_dictionary_string(apply_current_input_to_value, disp_string1, slen1);
        acknowledge(disp_string1, var_id); 
  
        do
        {
            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);

            send(45, status);

            assign_float(PV.DAQ.ANALOG_VALUE, 4.0);

            if ((status[STATUS_RESPONSE_CODE] == 3 ) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_current_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        /*
         * 20 mA test
         */
        current = 20.0;

        send(2, status);     /*Test the device to see if all is ok */

        if ((status[STATUS_RESPONSE_CODE] != 0) && (status[STATUS_RESPONSE_CODE] != 8))
        {
            display_response_status(2, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        get_dictionary_string(apply_current_input_to_value, disp_string1, slen1);
        acknowledge(disp_string1, var_id);

        do
        {
            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);

            send(46, status);

            assign_float(PV.DAQ.ANALOG_VALUE, 20.0);

            if ((status[STATUS_RESPONSE_CODE] == 3 ) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_current_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        process_abort();
    }
}

METHOD leave_fixed_current_mode
{
    LABEL [leaving_fixed_current_mode];
    HELP [leaving_fixed_current_mode_help_2];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        long var_id[1];
        int slen1;

        slen1 = 60;

        IGNORE_ALL_RESPONSE_CODES();

        assign_float(PV.DAQ.ANALOG_VALUE, 0.0);
        
        do
        {
            get_dictionary_string(return_fld_dev_to_orig_otpt, disp_string1, slen1);
            DELAY(3, disp_string1);

            send(40, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(40, status[STATUS_RESPONSE_CODE]);
            }

        } while (status[STATUS_RESPONSE_CODE]);

        get_dictionary_string(loop_may_be_returned_to_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.4
 * Device management Commands
 *
 * Commands 38, 41, 42, 48
 */

COMMAND reset_configuration_change_flag
{
    NUMBER 38;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR, [in_write_protect_mode];
        16, MODE_ERROR, [access_restricted];
    }
}

COMMAND perform_self_test  /* took out transmitter */
{
    NUMBER 41;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        16, MODE_ERROR, [access_restricted];
        32, MODE_ERROR, [busy];
    }
}

COMMAND perform_device_reset
{
    NUMBER 42;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        16, MODE_ERROR, [access_restricted];
        32, MODE_ERROR, [busy];
    }
}

VARIABLE device_specific_status_0
{
    LABEL [device_specific_status_0];
    HELP [field_device_status_0_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat0_1], MISC }, { 0x02, [fld_dev_stat0_2], MISC }, { 0x04, [fld_dev_stat0_3], MISC },
        { 0x08, [fld_dev_stat0_4], MISC }, { 0x10, [fld_dev_stat0_5], MISC }, { 0x20, [fld_dev_stat0_6], MISC },
        { 0x40, [fld_dev_stat0_7], MISC }, { 0x80, [fld_dev_stat0_8], MISC }
    }
}

VARIABLE device_specific_status_1
{
    LABEL [device_specific_status_1];
    HELP [field_device_status_1_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat1_1], MISC }, { 0x02, [fld_dev_stat1_2], MISC }, { 0x04, [fld_dev_stat1_3], MISC },
        { 0x08, [fld_dev_stat1_4], MISC }, { 0x10, [fld_dev_stat1_5], MISC }, { 0x20, [fld_dev_stat1_6], MISC },
        { 0x40, [fld_dev_stat1_7], MISC }, { 0x80, [fld_dev_stat1_8], MISC }
    }
}

VARIABLE device_specific_status_2
{
    LABEL [device_specific_status_2];
    HELP [field_device_status_2_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat2_1], MISC }, { 0x02, [fld_dev_stat2_2], MISC }, { 0x04, [fld_dev_stat2_3], MISC },
        { 0x08, [fld_dev_stat2_4], MISC }, { 0x10, [fld_dev_stat2_5], MISC }, { 0x20, [fld_dev_stat2_6], MISC },
        { 0x40, [fld_dev_stat2_7], MISC }, { 0x80, [fld_dev_stat2_8], MISC }
    }
}

VARIABLE device_specific_status_3
{
    LABEL [device_specific_status_3];
    HELP [field_device_status_3_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat3_1], MISC }, { 0x02, [fld_dev_stat3_2], MISC }, { 0x04, [fld_dev_stat3_3], MISC },
        { 0x08, [fld_dev_stat3_4], MISC }, { 0x10, [fld_dev_stat3_5], MISC }, { 0x20, [fld_dev_stat3_6], MISC },
        { 0x40, [fld_dev_stat3_7], MISC }, { 0x80, [fld_dev_stat3_8], MISC }
    }
}

VARIABLE device_specific_status_4
{
    LABEL [device_specific_status_4];
    HELP [field_device_status_4_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat4_1], MISC }, { 0x02, [fld_dev_stat4_2], MISC }, { 0x04, [fld_dev_stat4_3], MISC },
        { 0x08, [fld_dev_stat4_4], MISC }, { 0x10, [fld_dev_stat4_5], MISC }, { 0x20, [fld_dev_stat4_6], MISC },
        { 0x40, [fld_dev_stat4_7], MISC }, { 0x80, [fld_dev_stat4_8], MISC }
    }
}

VARIABLE device_specific_status_5
{
    LABEL [device_specific_status_5];
    HELP [field_device_status_5_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat5_1], MISC }, { 0x02, [fld_dev_stat5_2], MISC }, { 0x04, [fld_dev_stat5_3], MISC },
        { 0x08, [fld_dev_stat5_4], MISC }, { 0x10, [fld_dev_stat5_5], MISC }, { 0x20, [fld_dev_stat5_6], MISC },
        { 0x40, [fld_dev_stat5_7], MISC }, { 0x80, [fld_dev_stat5_8], MISC }
    }
}

VARIABLE device_specific_status_14
{
    LABEL [device_specific_status_14];
    HELP [field_device_status_14_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat14_1], MISC }, { 0x02, [fld_dev_stat14_2], MISC }, { 0x04, [fld_dev_stat14_3], MISC },
        { 0x08, [fld_dev_stat14_4], MISC }, { 0x10, [fld_dev_stat14_5], MISC }, { 0x20, [fld_dev_stat14_7], MISC },
        { 0x40, [fld_dev_stat14_7], MISC }, { 0x80, [fld_dev_stat14_8], MISC }
    }
}

VARIABLE device_specific_status_15
{
    LABEL [device_specific_status_15];
    HELP [field_device_status_15_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat15_1], MISC }, { 0x02, [fld_dev_stat15_2], MISC }, { 0x04, [fld_dev_stat15_3], MISC },
        { 0x08, [fld_dev_stat15_4], MISC }, { 0x10, [fld_dev_stat15_5], MISC }, { 0x20, [fld_dev_stat15_6], MISC },
        { 0x40, [fld_dev_stat15_7], MISC }, { 0x80, [fld_dev_stat15_8], MISC }
    }
}

VARIABLE device_specific_status_16
{
    LABEL [device_specific_status_16];
    HELP [field_device_status_16_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat16_1], MISC }, { 0x02, [fld_dev_stat16_2], MISC }, { 0x04, [fld_dev_stat16_3], MISC },
        { 0x08, [fld_dev_stat16_4], MISC }, { 0x10, [fld_dev_stat16_5], MISC }, { 0x20, [fld_dev_stat16_6], MISC },
        { 0x40, [fld_dev_stat16_7], MISC }, { 0x80, [fld_dev_stat16_8], MISC }
    }
}

VARIABLE device_specific_status_17
{
    LABEL [device_specific_status_17];
    HELP [field_device_status_17_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat17_1], MISC }, { 0x02, [fld_dev_stat17_2], MISC }, { 0x04, [fld_dev_stat17_3], MISC },
        { 0x08, [fld_dev_stat17_4], MISC }, { 0x10, [fld_dev_stat17_5], MISC }, { 0x20, [fld_dev_stat17_6], MISC },
        { 0x40, [fld_dev_stat17_7], MISC }, { 0x80, [fld_dev_stat17_8], MISC }
    }
}

VARIABLE device_specific_status_18
{
    LABEL [device_specific_status_18];
    HELP [field_device_status_18_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat18_1], MISC }, { 0x02, [fld_dev_stat18_2], MISC }, { 0x04, [fld_dev_stat18_3], MISC },
        { 0x08, [fld_dev_stat18_4], MISC }, { 0x10, [fld_dev_stat18_5], MISC }, { 0x20, [fld_dev_stat18_6], MISC },
        { 0x40, [fld_dev_stat18_7], MISC }, { 0x80, [fld_dev_stat18_8], MISC }
    }
}

VARIABLE device_specific_status_19
{
    LABEL [device_specific_status_19];
    HELP [field_device_status_19_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat19_1], MISC }, { 0x02, [fld_dev_stat19_2], MISC }, { 0x04, [fld_dev_stat19_3], MISC },
        { 0x08, [fld_dev_stat19_4], MISC }, { 0x10, [fld_dev_stat19_5], MISC }, { 0x20, [fld_dev_stat19_6], MISC },
        { 0x40, [fld_dev_stat19_7], MISC }, { 0x80, [fld_dev_stat19_8], MISC }
    }
}

VARIABLE device_specific_status_20
{
    LABEL [device_specific_status_20];
    HELP [field_device_status_20_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat20_1], MISC }, { 0x02, [fld_dev_stat20_2], MISC }, { 0x04, [fld_dev_stat20_3], MISC },
        { 0x08, [fld_dev_stat20_4], MISC }, { 0x10, [fld_dev_stat20_5], MISC }, { 0x20, [fld_dev_stat20_6], MISC },
        { 0x40, [fld_dev_stat20_7], MISC }, { 0x80, [fld_dev_stat20_8], MISC }
    }
}

VARIABLE device_specific_status_21
{
    LABEL [device_specific_status_21];
    HELP [field_device_status_21_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat21_1], MISC }, { 0x02, [fld_dev_stat21_2], MISC }, { 0x04, [fld_dev_stat21_3], MISC },
        { 0x08, [fld_dev_stat21_4], MISC }, { 0x10, [fld_dev_stat21_5], MISC }, { 0x20, [fld_dev_stat21_6], MISC },
        { 0x40, [fld_dev_stat21_7], MISC }, { 0x80, [fld_dev_stat21_8], MISC }
    }
}

VARIABLE device_specific_status_22
{
    LABEL [device_specific_status_22];
    HELP [field_device_status_22_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat22_1], MISC }, { 0x02, [fld_dev_stat22_2], MISC }, { 0x04, [fld_dev_stat22_3], MISC },
        { 0x08, [fld_dev_stat22_4], MISC }, { 0x10, [fld_dev_stat22_5], MISC }, { 0x20, [fld_dev_stat22_6], MISC },
        { 0x40, [fld_dev_stat22_7], MISC }, { 0x80, [fld_dev_stat22_8], MISC }
    }
}

VARIABLE device_specific_status_23
{
    LABEL [device_specific_status_23];
    HELP [field_device_status_23_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat23_1], MISC }, { 0x02, [fld_dev_stat23_2], MISC }, { 0x04, [fld_dev_stat23_3], MISC },
        { 0x08, [fld_dev_stat23_4], MISC }, { 0x10, [fld_dev_stat23_5], MISC }, { 0x20, [fld_dev_stat23_6], MISC },
        { 0x40, [fld_dev_stat23_7], MISC }, { 0x80, [fld_dev_stat23_8], MISC }
    }
}

VARIABLE device_specific_status_24
{
    LABEL [device_specific_status_24];
    HELP [field_device_status_24_help];
    CLASS DEVICE & DIAGNOSTIC & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
        { 0x01, [fld_dev_stat24_1], MISC }, { 0x02, [fld_dev_stat24_2], MISC }, { 0x04, [fld_dev_stat24_3], MISC },
        { 0x08, [fld_dev_stat24_4], MISC }, { 0x10, [fld_dev_stat24_5], MISC }, { 0x20, [fld_dev_stat24_6], MISC },
        { 0x40, [fld_dev_stat24_7], MISC }, { 0x80, [fld_dev_stat24_8], MISC }
    }
}

/*****************************************************************************
 * Command 48
 *
 * This is a generic definition of command 48 and most implementations
 * truncate the command when IMPORTING
 * 
 * NOTE 1 - the VARIABLE's device_specific_statusXX are normally REDEFINE'd by the
 * device DD 
 *
 * NOTE 2 - All devices must support at least bytes 0-7 including extended_fld_device_status
 * and operatingMode (which must be 0x00).  Devices requiring more than 6 bytes of device specific status
 * must include the analog channel saturated and fixed bytes.
 */ 
COMMAND read_additional_device_status
{
    NUMBER 48;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, 
            device_specific_status_0, device_specific_status_1, device_specific_status_2,
            device_specific_status_3, device_specific_status_4, device_specific_status_5,

            0x00, 0x00,

            0x00, 0x00,  analog_channel_saturated1,
            0x00, 0x00,  analog_channel_fixed1,

            device_specific_status_14, device_specific_status_15, device_specific_status_16,
            device_specific_status_17, device_specific_status_18, device_specific_status_19,
            device_specific_status_20, device_specific_status_21, device_specific_status_22,
            device_specific_status_23, device_specific_status_24
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,      [no_command_specific_errors];
        6,  MISC_ERROR,   [xmtr_specific_command_error];
        8,  MISC_WARNING, [update_in_progress];
        16, MODE_ERROR,   [access_restricted];
    }
}

/*
 * Methods for self test, device reset
 */

METHOD device_self_test 
{
    LABEL [selftest];
    HELP [selftest_help];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char xmtr_status[MAX_XMTR_STATUS_LEN];
        char disp_string1[60];             
        long x;
        int number_of_tries;
        int slen1;

        slen1 = 60;

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1); 
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(about_to_perform_limited_selftest, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(performing_selftest, disp_string1, slen1);
        PUT_MESSAGE(disp_string1);

        send(41, status);

        if(status[STATUS_RESPONSE_CODE])
        {
            display_response_status(41, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);             
            DELAY(2, disp_string1);
            process_abort();
        }

        RETRY_ON_NO_DEVICE();
        number_of_tries = 0;

        do
        {
            get_more_status(status, xmtr_status);
        } while (((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
                || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
                && (number_of_tries++ < MAXIMUM_RESET_RETRIES));

        if ((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
                || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
        {
            get_dictionary_string(device_stuck_in_self_test, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }
        else if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(48, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);             
            DELAY(2, disp_string1);
        }
        else if (xmtr_status[0])
        {
            for (x=1; x < 0x100; x <<= 1)
            {
                if (xmtr_status[0] & x)
                {
                    display_xmtr_status(device_specific_status_0, x);
                }
            }
        }
        else
        {
            get_dictionary_string(self_test_ok, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }

        process_abort();        /* this will call return_to_normal */
    }
}

METHOD device_master_reset 
{
    LABEL [device_reset];
    HELP [device_reset_help];
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char xmtr_status[MAX_XMTR_STATUS_LEN];
        char disp_string1[60];             
        long x;
        int number_of_tries;
        int slen1;

        slen1 = 60;

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1); 
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(about_to_perform_device_reset, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(device_reset_in_progress, disp_string1, slen1);
        PUT_MESSAGE(disp_string1);

        send(42, status);

        if(status[STATUS_RESPONSE_CODE])
        {
            display_response_status(42, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);             
            DELAY(2, disp_string1);
            process_abort();
        }

        RETRY_ON_NO_DEVICE();
        number_of_tries = 0;

        do 
        {
            get_more_status(status, xmtr_status);
        } while (((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
                || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
                && (number_of_tries++ < MAXIMUM_RESET_RETRIES));

        if ((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
                || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
        {
            get_dictionary_string(device_stuck_in_device_reset, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }
        else if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(48, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);             
            DELAY(2, disp_string1);
        }
        else if (xmtr_status[0])
        {
            for (x=1; x < 0x100; x <<= 1)
            {
                if (xmtr_status[0] & x)
                {
                    display_xmtr_status(device_specific_status_0, x);
                }
            }
        }
        else
        {
            get_dictionary_string(device_reset_ok, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }

        process_abort();        /* this will call return_to_normal */
    }
}

METHOD zero_trim
{
    LABEL [zero_trim];
    HELP [zero_trim_help];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        int retry;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        add_abort_method(return_to_normal);

        get_dictionary_string(this_will_affect_snsr_cal, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        do
        {
            get_dictionary_string(apply_0_input_to_snsr, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);

            get_dictionary_string(snsr_input_stabilizing, disp_string1, slen1);
            DELAY(3, disp_string1);

            send(43, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(43, status[STATUS_RESPONSE_CODE]);

                if ((status[STATUS_RESPONSE_CODE] != 9)
                        && (status[STATUS_RESPONSE_CODE] != 10))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }

                get_dictionary_string(zero_the_input_and_try_again, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = SELECT_FROM_LIST(disp_string1, disp_string2);
            }
            else
            {
                get_dictionary_string(snsr_zero_succeeded, disp_string1, slen1);
                DELAY(3, disp_string1);
                process_abort();
            }
        } while (!retry);

        process_abort();
    }
}

METHOD return_to_normal
{
    LABEL [aborting_return_to_normal];
    DEFINITION
    {
        char disp_string1[60];
        int slen1;

        slen1 = 60;

        get_dictionary_string(loop_may_be_returned_to_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}

METHOD warning_message
{
    LABEL [set_loop_to_manual];
    DEFINITION
    {
        char disp_string1[125];
        int slen1;

        slen1 = 125;

        get_dictionary_string(remove_from_auto_befor_send, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}

METHOD device_variable_zero_trim
{
    LABEL [field_device_variable_zero_trim];
    HELP [zero_trim_help];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        int tv;
        int retry;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        add_abort_method(return_to_normal);

        get_dictionary_string(this_will_affect_snsr_cal, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(select_fld_dev_var_snsr_trim, disp_string1, slen1);
        GET_DEV_VAR_VALUE(disp_string1, device_variable_code);

        do
        {
            get_dictionary_string(apply_0_input_to_snsr, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);

            get_dictionary_string(snsr_input_stabilizing, disp_string1, slen1);
            DELAY(3, disp_string1);

            send(52, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(52, status[STATUS_RESPONSE_CODE]);

                if ((status[STATUS_RESPONSE_CODE] != 2)
                        && (status[STATUS_RESPONSE_CODE] != 9)
                        && (status[STATUS_RESPONSE_CODE] != 10))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }

                get_dictionary_string(zero_the_input_and_try_again, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);

                retry = SELECT_FROM_LIST(disp_string1, disp_string2);

                if (!retry)
                {
                    if (status[STATUS_RESPONSE_CODE] == 2)
                    {
                        do
                        {
                            tv = ivar_value(device_variable_code);
                            get_dictionary_string(select_fld_dev_var_snsr_trim, disp_string1, slen1);

                            GET_DEV_VAR_VALUE(disp_string1, device_variable_code);

                            if (tv == ivar_value(device_variable_code))
                            {
                                get_dictionary_string(invalid_value_try_again, disp_string1, slen1);
                                DELAY(2, disp_string1);
                            }
                        } while (tv == ivar_value(device_variable_code));
                    }
                }
            }
            else
            {
                get_dictionary_string(snsr_zero_succeeded, disp_string1, slen1);
                DELAY(3, disp_string1);
                process_abort();
            }
        } while (!retry);

        process_abort();
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.6
 * Device Variable Mapping Commands
 *
 * Commands 50 51
 */

VARIABLE primary_variable_code
{
    LABEL [primary_variable_is];
    HELP [primary_variable_code_is_help];
    CLASS HART;
    TYPE INDEX deviceVariables;
    POST_EDIT_ACTIONS
    {
        warning_message
    }
}

VARIABLE secondary_variable_code
{
    LABEL [secondary_variable_is];
    HELP [secondary_variable_code_is_help];
    CLASS HART;
    TYPE INDEX deviceVariables;
    POST_EDIT_ACTIONS
    {
        warning_message
    }
}

VARIABLE tertiary_variable_code
{
    LABEL [tertiary_variable_is];
    HELP [tertiary_variable_code_is_help];
    CLASS HART;
    TYPE INDEX deviceVariables;
    POST_EDIT_ACTIONS
    {
        warning_message
    }
}

VARIABLE quaternary_variable_code
{
    LABEL [quaternary_variable_is];
    HELP [quaternary_variable_code_is_help];
    CLASS HART;
    TYPE INDEX deviceVariables;
    POST_EDIT_ACTIONS
    {
        warning_message
    }
}

COMMAND read_device_variable_assignments
{
    NUMBER 50;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            primary_variable_code, secondary_variable_code,
            tertiary_variable_code, quaternary_variable_code
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        16, MODE_ERROR, [access_restricted];
    }
}

COMMAND write_device_variable_assignments
{
    NUMBER 51;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            primary_variable_code, secondary_variable_code,
            tertiary_variable_code, quaternary_variable_code
        }
        REPLY
        {
            response_code, device_status,
            primary_variable_code, secondary_variable_code,
            tertiary_variable_code, quaternary_variable_code
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.7
 * PV Commands
 *
 * Commands 34 43 44 47 49
 */

COMMAND write_pv_damping_value
{
    NUMBER 34;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.DEVICE_VARIABLE.DAMPING_VALUE
        }
        REPLY
        {
            response_code, device_status, PV.DEVICE_VARIABLE.DAMPING_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR,   [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR,   [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        8,  DATA_ENTRY_WARNING, [set_to_nearest_possible_value];
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
    }
}

COMMAND set_pv_zero
{
    NUMBER 43;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,       [no_command_specific_errors];
        6,  MISC_ERROR,    [xmtr_specific_command_error];
        7,  MODE_ERROR,    [in_write_protect_mode];
        9,  PROCESS_ERROR, [applied_process_too_high];
        10, PROCESS_ERROR, [applied_process_too_low];
        16, MODE_ERROR,    [access_restricted];
        32, MODE_ERROR,    [busy];
    }
}

COMMAND write_pv_units
{
    NUMBER 44;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.DEVICE_VARIABLE.DIGITAL_UNITS
        }
        REPLY
        {
            response_code, device_status, PV.DEVICE_VARIABLE.DIGITAL_UNITS
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND write_pv_transfer_function
{
    NUMBER 47;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.RANGING.TRANSFER_FUNCTION
        }
        REPLY
        {
            response_code, device_status, PV.RANGING.TRANSFER_FUNCTION
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND write_pv_sensor_serial_number
{
    NUMBER 49;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.DEVICE_VARIABLE.SENSOR_SERIAL_NUMBER
        }
        REPLY
        {
            response_code, device_status, PV.DEVICE_VARIABLE.SENSOR_SERIAL_NUMBER
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}


/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.8
 * Device Variable Commands.  The commands allow access to the Device
 * Variables in the Field Device
 *
 * Commands 33, 52-56, 79, 113, 114
 *
 */

COMMAND read_device_variables
{
    NUMBER 33;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code_1 (INFO,INDEX),
            device_variable_code_2 (INFO,INDEX),
            device_variable_code_3 (INFO,INDEX),
            device_variable_code_4 (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            device_variable_code_1 (INFO,INDEX),
            deviceVariables[device_variable_code_1].DIGITAL_UNITS,
            deviceVariables[device_variable_code_1].DIGITAL_VALUE,
            device_variable_code_2 (INFO,INDEX),
            deviceVariables[device_variable_code_2].DIGITAL_UNITS,
            deviceVariables[device_variable_code_2].DIGITAL_VALUE,
            device_variable_code_3 (INFO,INDEX),
            deviceVariables[device_variable_code_3].DIGITAL_UNITS,
            deviceVariables[device_variable_code_3].DIGITAL_VALUE,
            device_variable_code_4 (INFO,INDEX),
            deviceVariables[device_variable_code_4].DIGITAL_UNITS,
            deviceVariables[device_variable_code_4].DIGITAL_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        8,  MISC_WARNING,     [update_failure];
        16, MODE_ERROR,       [access_restricted];
    }
}

VARIABLE device_variable_code
{
    LABEL "Device Variable";
    HELP [index_into_dynamic_variables_array_help];
/*  APPHELP "This variable can represent measured or calculated type variables."
            "The numeric assigned to a Field Device variable can never be"
            "changed. If it is changed this would be a violation of the Forward"
            "Compatibility rules."
            "HART_VAR:XMTR VAR CODE"; */
    CLASS LOCAL;
    TYPE INDEX deviceVariables;
}

COMMAND set_device_variable_zero
{
    NUMBER 52;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code
        }
        REPLY
        {
            response_code, device_status, device_variable_code
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        9,  PROCESS_ERROR,    [applied_process_too_high];
        10, PROCESS_ERROR,    [applied_process_too_low];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND write_device_variable_units
{
    NUMBER 53;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].DIGITAL_UNITS
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].DIGITAL_UNITS
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        11, DATA_ENTRY_ERROR, [invalid_device_variable];
        12, DATA_ENTRY_ERROR, [invalid_units];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND read_device_variable_information
{
    NUMBER 54;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].SENSOR_SERIAL_NUMBER,
            deviceVariables[device_variable_code].DIGITAL_UNITS,
            deviceVariables[device_variable_code].UPPER_SENSOR_LIMIT,
            deviceVariables[device_variable_code].LOWER_SENSOR_LIMIT,
            deviceVariables[device_variable_code].DAMPING_VALUE,
            deviceVariables[device_variable_code].MINIMUM_SPAN
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND write_device_variable_damping_value
{
    NUMBER 55;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].DAMPING_VALUE
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].DAMPING_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR,   [invalid_selection];
        3,  DATA_ENTRY_ERROR,   [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR,   [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        8,  DATA_ENTRY_WARNING, [set_to_nearest_possible_value];
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
    }
}

COMMAND write_device_variable_sensor_serial_number
{
    NUMBER 56;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].SENSOR_SERIAL_NUMBER
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].SENSOR_SERIAL_NUMBER
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.9
 * Burst mode Commands
 *
 * Commands 105, 107-109
 */

VARIABLE burst_command_number
{
    LABEL [burst_command_number];
    HELP [burst_command_number_help];
    CLASS HART;
    TYPE ENUMERATED
    { 
        {1,  [read_primary_variable], [pv_help] }, 
        {2,  [read_percent_range_and_current], [percent_rnge_help] }, 
        {3,  [read_dyn_vars_and_current], [all_process_var_help] } 
    }
}

VARIABLE burst_variable_code_1
{
    LABEL [burst_variable_1];
    HELP [burst_variable_1_help];
    CLASS HART;
    TYPE INDEX deviceVariables;
}

VARIABLE burst_variable_code_2
{
    LABEL [burst_variable_2];
    HELP [burst_variable_2_help];
    CLASS HART;
    TYPE INDEX deviceVariables;
}

VARIABLE burst_variable_code_3
{
    LABEL [burst_variable_3];
    HELP [burst_variable_3_help];
    CLASS HART;
    TYPE INDEX deviceVariables;
}

VARIABLE burst_variable_code_4
{
    LABEL [burst_variable_4];
    HELP [burst_variable_4_help];
    CLASS HART;
    TYPE INDEX deviceVariables;
}

COMMAND write_burst_device_variables
{
    NUMBER 107;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            burst_variable_code_1, burst_variable_code_2,
            burst_variable_code_3, burst_variable_code_4
        }
        REPLY
        {
            response_code, device_status,
            burst_variable_code_1, burst_variable_code_2,
            burst_variable_code_3, burst_variable_code_4
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
    }
}

COMMAND write_burst_command_number
{
    NUMBER 108;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            burst_command_number
        }

        REPLY
        {
            response_code, device_status, burst_command_number
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
    }
}

COMMAND burst_mode_control
{
    NUMBER 109;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            burst_mode_select
        }
        REPLY
        {
            response_code, device_status, burst_mode_select
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.10
 * Analog channel Commands
 *
 * Commands 60-70
 */

/*
 *****************************************************************************
 * Variable  analog_channel_number_code is used to specify the analog channel
 * for the following commands
 *
 * NOTE - Index = 0 is Primary Variable
 */
VARIABLE analog_channel_number_code
{
    LABEL "Analog Channel Number";
    HELP "Analog Channel Number- Index into an array of Analog Channels.";
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

COMMAND read_analog_output_and_percent_of_range
{
    NUMBER 60;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE,
            dynamic_variables[analog_channel_number_code].RANGING.PERCENT_RANGE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        8,  MISC_WARNING,     [update_failure];
        16, MODE_ERROR,       [access_restricted];
    }
}

VARIABLE analog_channel_number_code_1
{
    LABEL "Index 1 into Analog Channel Array";
    HELP "Index into Analog Channel Array- Is the first of an array of addresses, whose"
         " value is identified by the Analog Channel Code.";
/*  APPHELP "This variable is used to access more variables that relate to"
            " that Analog Output index."
            " NOTE - Index = 0 is not valid"
            " HART_VAR:SLOT #0 ANALOG OUTPUT NUMBER CODE"; */
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

VARIABLE analog_channel_number_code_2
{
    LABEL "Index 2 into Analog Channel Array";
    HELP "Index into Analog Channel Array- Is the second of an array of addresses, whose"
         " value is identified by the Analog Channel Code.";
/*  APPHELP "This variable is used to access more variables that relate to"
            " that Analog Output index."
            " NOTE - Index = 0 is not valid"
            " HART_VAR:SLOT #1 ANALOG OUTPUT NUMBER CODE"; */
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

VARIABLE analog_channel_number_code_3
{
    LABEL "Index 3 into Analog Channel Array";
    HELP "Index into Analog Channel Array- Is the third of an array of addresses, whose"
         " value is identified by the Analog Channel Code.";
/*  APPHELP "This variable is used to access more variables that relate to"
            " that Analog Output index."
            " NOTE - Index = 0 is not valid"
            " HART_VAR:SLOT #2 ANALOG OUTPUT NUMBER CODE"; */
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

VARIABLE analog_channel_number_code_4
{
    LABEL "Index 4 into Analog Channel Array";
    HELP "Index into Analog Channel Array- Is the fourth of an array of addresses, whose"
         " value is identified by the Analog Channel Code.";
/*  APPHELP "This variable is used to access more variables that relate to"
            " that Analog Output index."
            " NOTE - Index = 0 is not valid"
            " HART_VAR:SLOT #3 ANALOG OUTPUT NUMBER CODE"; */
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

COMMAND read_analog_channels
{
    NUMBER 62;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code_1 (INFO,INDEX), analog_channel_number_code_2 (INFO,INDEX),
            analog_channel_number_code_3 (INFO,INDEX), analog_channel_number_code_4 (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code_1 (INFO,INDEX),
            dynamic_variables[analog_channel_number_code_1].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code_1].DAQ.ANALOG_VALUE,
            analog_channel_number_code_2 (INFO,INDEX),
            dynamic_variables[analog_channel_number_code_2].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code_2].DAQ.ANALOG_VALUE,
            analog_channel_number_code_3 (INFO,INDEX),
            dynamic_variables[analog_channel_number_code_3].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code_3].DAQ.ANALOG_VALUE,
            analog_channel_number_code_4 (INFO,INDEX),
            dynamic_variables[analog_channel_number_code_4].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code_4].DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        8,  MISC_WARNING,     [update_failure];
    }
}

COMMAND read_analog_channel_information
{
    NUMBER 63;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ALARM_CODE,
            dynamic_variables[analog_channel_number_code].RANGING.TRANSFER_FUNCTION,
            dynamic_variables[analog_channel_number_code].RANGING.RANGE_UNITS,
            dynamic_variables[analog_channel_number_code].RANGING.UPPER_RANGE_VALUE,
            dynamic_variables[analog_channel_number_code].RANGING.LOWER_RANGE_VALUE,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_DAMPING
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND write_analog_channel_additional_damping_value
{
    NUMBER 64;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_DAMPING
        }
        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_DAMPING
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR,   [invalid_selection];
        3,  DATA_ENTRY_ERROR,   [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR,   [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        8,  DATA_ENTRY_WARNING, [set_to_nearest_possible_value];
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
    }
}

COMMAND write_analog_channel_range_values
{
    NUMBER 65;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].RANGING.RANGE_UNITS (INFO),
            dynamic_variables[analog_channel_number_code].RANGING.UPPER_RANGE_VALUE,
            dynamic_variables[analog_channel_number_code].RANGING.LOWER_RANGE_VALUE
        }
        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].RANGING.RANGE_UNITS (INFO),
            dynamic_variables[analog_channel_number_code].RANGING.UPPER_RANGE_VALUE,
            dynamic_variables[analog_channel_number_code].RANGING.LOWER_RANGE_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        9,  DATA_ENTRY_ERROR,   [lower_range_value_too_high];
        10, DATA_ENTRY_ERROR,   [lower_range_value_too_low];
        11, DATA_ENTRY_ERROR,   [upper_range_value_too_high];
        12, DATA_ENTRY_ERROR,   [upper_range_value_too_low];
        13, DATA_ENTRY_ERROR,   [URV_and_LRV_out_of_limits];
        14, DATA_ENTRY_WARNING, [span_too_small];
        15, DATA_ENTRY_ERROR,   [invalid_analog_output_number_code];
        16, MODE_ERROR,         [access_restricted];
        28, DATA_ENTRY_ERROR,   [invalid_units];
        32, MODE_ERROR,         [busy];
    }
}

COMMAND enter_exit_fixed_analog_channel_mode
{
    NUMBER 66;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        11, MODE_ERROR,       [in_multidrop_mode];
        12, DATA_ENTRY_ERROR, [invalid_units];
        15, DATA_ENTRY_ERROR, [invalid_analog_output_number_code];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND trim_analog_channel_zero
{
    NUMBER 67;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        9,  MODE_ERROR,       [not_in_proper_anlg_output_mode];
        11, MODE_ERROR,       [in_multidrop_mode];
        12, DATA_ENTRY_ERROR, [invalid_units];
        15, DATA_ENTRY_ERROR, [invalid_analog_output_number_code];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND trim_analog_channel_gain
{
    NUMBER 68;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        9,  MODE_ERROR,       [not_in_proper_anlg_output_mode];
        11, MODE_ERROR,       [in_multidrop_mode];
        12, DATA_ENTRY_ERROR, [invalid_units];
        15, DATA_ENTRY_ERROR, [invalid_analog_output_number_code];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND write_analog_channel_transfer_function
{
    NUMBER 69;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].RANGING.TRANSFER_FUNCTION
        }

        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].RANGING.TRANSFER_FUNCTION
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        13, DATA_ENTRY_ERROR, [invalid_transfer_function_code];
        15, DATA_ENTRY_ERROR, [invalid_analog_output_number_code];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND read_analog_channel_endpoint_values
{
    NUMBER 70;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status, analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.UPPER_ENDPOINT_VALUE,
            dynamic_variables[analog_channel_number_code].DAQ.LOWER_ENDPOINT_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

METHOD fixed_analog_output
{
    LABEL [fix_analog_output];
    HELP [fix_analog_output_help];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[3];
        int use_abort_method;
        int choice;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        use_abort_method = 0;

        get_dictionary_string(select_analog_output_to_trim, disp_string1, slen1);
        GET_DEV_VAR_VALUE(disp_string1, analog_channel_number_code);

        get_dictionary_string(select_analog_output_units, disp_string1, slen1);
        GET_DEV_VAR_VALUE(disp_string1,dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS);

        var_id[0] = VARID(analog_channel_number_code);
        var_id[2] = VARID(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS);

        do
        {
            if (ivar_value(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS)== 39)
            {
                get_dictionary_string(choose_analog_output_level, disp_string1, slen1);
                get_dictionary_string(choices_4_20_other_end, disp_string2, slen2);

                choice = SELECT_FROM_LIST(disp_string1, disp_string2);

                switch (choice)
                {
                case 0:
                    assign_float(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE, 4.0);
                    break;

                case 1:
                    assign_float(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE, 20.0);
                    break;

                case 2:
                    get_dictionary_string(enter_desired_analog_output, disp_string1, slen1);
                    GET_DEV_VAR_VALUE(disp_string1,
                            dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);
                    break;

                case 3:
                default:
                    process_abort();
                    break;
                }
            }
            else
            {
                get_dictionary_string(choose_analog_output_level, disp_string1, slen1);
                get_dictionary_string(new_value_end, disp_string2, slen2);

                choice = SELECT_FROM_LIST(disp_string1, disp_string2);

                switch(choice)
                {
                case 0:
                    get_dictionary_string(enter_desired_analog_output, disp_string1, slen1);
                    GET_DEV_VAR_VALUE(disp_string1,
                            dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);
                    break;
                case 1:
                    process_abort();
                    break;
                }
            }

            send(66,status);

            if (!use_abort_method)
            {
                if ((status[STATUS_RESPONSE_CODE] != 0)
                        && (status[STATUS_RESPONSE_CODE] != 3)
                        && (status[STATUS_RESPONSE_CODE] != 4))
                {
                    display_response_status(66, status[STATUS_RESPONSE_CODE]);
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }
                if (status[STATUS_RESPONSE_CODE] == 0)
                {
                    use_abort_method = 1;
                    remove_abort_method(return_to_normal);
                    add_abort_method(leave_analog_fixed_current_mode);
                }
            }

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(66, status[STATUS_RESPONSE_CODE]);
                if ((status[STATUS_RESPONSE_CODE] != 3) && (status[STATUS_RESPONSE_CODE] != 4))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }
            }
            else
            {
                var_id[1] = VARID(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);
                get_dictionary_string(fld_dev_AO_is_fixed_at, disp_string1, slen1);
                acknowledge(disp_string1, var_id);
            }
        } while (choice < 3);
    }
}

METHOD analog_dac_trim
{
    LABEL [trim_analog_output];
    HELP [digital_to_analog_trim_help];
/*  APPHELP "This type of calibration is usually used to compensate for"
            " drifting or aging output electronics"; */
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[3];
        int retry;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        add_abort_method(return_to_normal);

        get_dictionary_string(connect_reference_meter, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(select_analog_output_to_trim,disp_string1,slen1);
        GET_DEV_VAR_VALUE(disp_string1, analog_channel_number_code);
        get_dictionary_string(select_analog_output_units, disp_string1, slen1);

        GET_DEV_VAR_VALUE(disp_string1,
            dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS);

        var_id[0] = VARID(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);
        var_id[1] = VARID(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS); 

        assign_var(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,
            dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.LOWER_ENDPOINT_VALUE);

        get_dictionary_string(about_to_set_fld_dev_output, disp_string1, slen1);
        acknowledge(disp_string1, var_id);

        send(66, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(66, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }
        else
        {
            remove_abort_method(return_to_normal);
            add_abort_method(leave_analog_fixed_current_mode);
        }

        do
        {
            assign_var(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,
                dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.LOWER_ENDPOINT_VALUE);

            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1,
                dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);

            send(67, status);

            if ((status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(67, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(67, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_output_eq_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        assign_var(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,
            dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.UPPER_ENDPOINT_VALUE);

        get_dictionary_string(about_to_set_fld_dev_output, disp_string1, slen1);
        acknowledge(disp_string1, var_id);

        send(66, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(66, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        do
        {
            assign_var(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,
                dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.UPPER_ENDPOINT_VALUE);

            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1,
                dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);

            send(68, status);
            if ((status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(68, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(68, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_output_eq_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        process_abort();
    }
}

METHOD leave_analog_fixed_current_mode
{
    LABEL [leaving_fixed_current_mode];
    HELP [leaving_fixed_current_mode_help_1];
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        long var_id[1];
        int slen1;

        slen1 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        assign_float(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,NaN);

        do
        {
            get_dictionary_string(return_fld_dev_to_orig_otpt, disp_string1, slen1);
            DELAY(3, disp_string1);

            send(66, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(66, status[STATUS_RESPONSE_CODE]);
            }

        } while(status[STATUS_RESPONSE_CODE]);

        get_dictionary_string(loop_may_be_returned_to_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}

/*
 *****************************************************************************
 *The Following commands are obsolete and NOT RECOMMENDED for new Designs
 *
 */

VARIABLE eeprom_select
{
    LABEL [EEPROM_select];
    HELP [eeprom_select_help];
/*  APPHELP "For Burn requests, the Burn may not begin until the response that"
            " acknowledges the receipt of the variable has been sent back to the"
            " Master."
            " HART_VAR:EEPROM CONTROL"; */
    CLASS LOCAL;
    TYPE ENUMERATED
    { 
        { 0, [burn_eeprom] }, 
        { 1, [restore_shadow_ram] }
    }
}

COMMAND eeprom_control
{
    NUMBER 39;
    OPERATION COMMAND;
    TRANSACTION 
    {
        REQUEST
        {
            eeprom_select
        }
        REPLY
        {
            response_code, device_status, eeprom_select
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

METHOD burn_eeprom
{
    LABEL [burn_eeprom];
    HELP [burn_eeprom_help];
    DEFINITION
    {
        char status[3];
        char disp_string1[60];
        int slen1;

        slen1 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        assign_int(eeprom_select,0);

        get_dictionary_string(making_changes_permanent, disp_string1, slen1);
        DELAY(3, disp_string1);

        send(39, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(39, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(previous_err_seen_brn_eeprom, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }
    }
}

VARIABLE unit_tag
{
    LABEL [unit_tag];
    HELP [unit_tag_help];
/*  APPHELP "A Field Device may have sub applications that require being"
            " addressed individually. For this addressing the Tag variable"
            " associated with Command #11 should be used. This Tag is used to"
            " identify the application that controls the sub applications. This"
            " Text can be used by the user in any way. A recommended use is as"
            " a unique label to a plant that correlates to a Field Device label:"
            " a plant drawing, or on a Control System."
            " NOTE - HART does not recommend using this variable. If you have an"
            " application that controls the sub applications, all of these"
            " applications should be addressed using the HART Frame itself."
            " HART_VAR:UNIT TAG"; */
    CLASS DEVICE;
    TYPE PACKED_ASCII (8);
}

VARIABLE unit_descriptor
{
    LABEL [unit_descriptor];
    HELP [unit_descriptor_help];
/*  APPHELP "A Field Device may have sub applications that require being"
            " addressed individually. This Descriptor"
            " is used to identify the application that controls the sub"
            " applications. This text can be used by the user in any way. There is"
            " no recommended use.
            " NOTE - HART does not recommend using this variable. If you have an"
            " application that controls the sub applications, all of these"
            " applications should be addressed using the HART Frame itself."
            " HART_VAR:UNIT DESCR"; */
    CLASS DEVICE;
    TYPE PACKED_ASCII (16);
}

VARIABLE unit_date
{
    LABEL [unit_date];
    HELP [unit_date_help];
/*  APPHELP "A Field Device may have sub applications that require being"
            " addressed individually. This Date is used to"
            " identify the application that controls the sub applications."
            " This text can be used by the user in any way. There is"
            " no recommended use.
            " Format: Byte Closest to the Delimiter: Day, Next Byte: Month,
            " Byte Furthest from the Delimiter: Year-1900.
            " NOTE - HART does not recommend using this variable. If you have an"
            " application that controls the sub applications, all of these"
            " applications should be addressed using the HART Frame itself."
            " HART_VAR:UNIT DATE"; */
    CLASS DEVICE;
    TYPE DATE;
}

COMMAND read_unit_tag_descriptor_date
{
    NUMBER 57;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            unit_tag, unit_descriptor, unit_date
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        16, MODE_ERROR, [access_restricted];
    }
}

COMMAND write_unit_tag_descriptor_date
{
    NUMBER 58;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            unit_tag, unit_descriptor, unit_date
        }
        REPLY
        {
            response_code, device_status,
            unit_tag, unit_descriptor, unit_date
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*****************************************************************************
 * The following commands are NOT RECOMMENDED FOR NEW DESIGNS
 */

COMMAND read_dynamic_variables_and_pv_analog_output
{
    NUMBER 61;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            PV.DAQ.ANALOG_UNITS, PV.DAQ.ANALOG_VALUE,
            PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE,
            SV.DEVICE_VARIABLE.DIGITAL_UNITS, SV.DEVICE_VARIABLE.DIGITAL_VALUE,
            TV.DEVICE_VARIABLE.DIGITAL_UNITS, TV.DEVICE_VARIABLE.DIGITAL_VALUE,
            QV.DEVICE_VARIABLE.DIGITAL_UNITS, QV.DEVICE_VARIABLE.DIGITAL_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,        [no_command_specific_errors];
        6,  MISC_ERROR,     [xmtr_specific_command_error];
        8,  MISC_WARNING,   [update_failure];
        16, MODE_ERROR,     [access_restricted];
    }
}

COMMAND read_all_dynamic_variables
{
    NUMBER 110;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE,
            SV.DEVICE_VARIABLE.DIGITAL_UNITS, SV.DEVICE_VARIABLE.DIGITAL_VALUE,
            TV.DEVICE_VARIABLE.DIGITAL_UNITS, TV.DEVICE_VARIABLE.DIGITAL_VALUE,
            QV.DEVICE_VARIABLE.DIGITAL_UNITS, QV.DEVICE_VARIABLE.DIGITAL_VALUE
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,        [no_command_specific_errors];
        6,  MISC_ERROR,     [xmtr_specific_command_error];
        7,  MODE_ERROR,     [in_write_protect_mode];
        8,  MISC_WARNING,   [update_failure];
        16, MODE_ERROR,     [access_restricted];
    }
}

